```

title = "Digitální teploměr s ATmega8"
perex = "Další jednoduché zapojení určené pro náš kroužek mládeže, tentokrát jednoduchého teploměru s ATmega8 a čidlem teploty DS18B20. Najdete zde kompletní popis programu a zapojení včetně plošného spoje."
tags = ["Články","Konstrukce","Konstrukce s mikroprocesorem"]
image = "lead.jpg"
lang = "c"
[author]
name = "Michal, OK1WMR"
[date]
creation="2012-06-15 13:56:00"
```

![Digitální teploměr s ATmega8](teplomer_1.jpg)

Další jednoduché zapojení určené pro náš kroužek mládeže, tentokrát jednoduchého teploměru s ATmega8 a čidlem teploty [DS18B20](https://datasheets.maximintegrated.com/en/ds/DS18B20.pdf). Najdete zde kompletní popis programu a zapojení včetně plošného spoje.

**Pro správnou funkci teploměru je zapotřebí naprogramovat pojistky mikroprocesoru (fuses).**
Pojistkami nastavíte, aby mikroprocesor použil jako hodinový signál interní 8 MHz oscilátor, jinak by běžěl na standardní frekvenci 1 MHz a nefungovalo by správně zobrazování teploty. Naše doporučené hodnoty pojistek jsou: `hfuse=D9, lfuse=04`.

# Schéma

![Schéma](teplomer_sch.png?width=600)

# Plošný spoj

![Plošný spoj](teplomer_pcb.png?width=600)

# Zdrojové soubory

**Schéma:** [eagle](https://git.ok1kvk.cz/krouzek/digitalni-teplomer-s-atmega8/raw/master/HW/digitalni-teplomer.sch),
[PDF](https://git.ok1kvk.cz/krouzek/digitalni-teplomer-s-atmega8/raw/master/digitalni-teplomer-schema.pdf)
**Plošný spoj:** [eagle](https://git.ok1kvk.cz/krouzek/digitalni-teplomer-s-atmega8/raw/master/HW/digitalni-teplomer.brd), [PDF 1:1](https://git.ok1kvk.cz/krouzek/digitalni-teplomer-s-atmega8/raw/master/plosny-spoj.pdf)

**Celý projekt:** [ZIP](https://git.ok1kvk.cz/krouzek/digitalni-teplomer-s-atmega8/repository/archive.zip?ref=master), [TAR.GZ](https://git.ok1kvk.cz/krouzek/digitalni-teplomer-s-atmega8/repository/archive.tar.gz?ref=master)

**Projekt na GitLabu** [digitalni-teplomer-s-atmega8.git](https://git.ok1kvk.cz/krouzek/digitalni-teplomer-s-atmega8)

Aktuální zdrojový kód naleznete na našem [GitLabu](https://git.ok1kvk.cz/krouzek/digitalni-teplomer-s-atmega8/blob/master/SW/hodiny.c), kde také naleznete [přeložený zdojový kód](https://git.ok1kvk.cz/krouzek/digitalni-teplomer-s-atmega8/builds) do formátu [HEX](https://cs.wikipedia.org/wiki/Intel_HEX).

Deska byla vyrobena [metodou nažehlením tonneru](/clanek/2008/vyroba-plosnych-spoju/).

# Seznam součástek:

| Označení | Hodnota            | Datasheet                                                                                               | Odkaz do obchodu                                                                                       |
| -------- | ------------------ | ------------------------------------------------------------------------------------------------------- | ------------------------------------------------------------------------------------------------------ |
| R1 - R8  | 330 &#8486;        |                                                                                                         |                                                                                                        |
| R9       | 10 k&#8486;        |                                                                                                         |                                                                                                        |
| R10      | 4.7 k&#8486;       |                                                                                                         |                                                                                                        |
| C1, C2   | 100 &#181;F        |                                                                                                         |                                                                                                        |
| C3       | 100 nF             |                                                                                                         |                                                                                                        |
| IC1      | 7805 (TO220)       | [LM7805.pdf](https://www.sparkfun.com/datasheets/Components/LM7805.pdf)                                 | [tme.eu](http://www.tme.eu/cz/details/l7805acv/stabilizatory-napeti-neregulovane/st-microelectronics/) |
| IC2      | ATmega8A-PU        | [ATmega8a.pdf](http://www.atmel.com/images/atmel-8159-8-bit-avr-microcontroller-atmega8a_datasheet.pdf) | [tme.eu](http://www.tme.eu/cz/details/atmega8a-pu/mikroprocesory-atmel-avr-tht/atmel/)                 |
| IC3      | DS18B20            | [DS18B20.pdf](http://www.tme.eu/cz/Document/dd11228ca818d1ed1f3029ea47fef609/DS18B20+.pdf)              | [tme.eu](http://www.tme.eu/cz/details/ds18b20+/prevodniky-teploty/maxim-dallas/)                       |
| D1       | LED displej        | [CA56-12SRWA.pdf](http://www.tme.eu/cz/Document/dd941439913e329281bd9cec3769262a/CA56-12SRWA.pdf)       | [tme.eu](http://www.tme.eu/cz/details/ca56-12srwa/ctyrmistne-led-displeje/kingbright-electronic/)      |
| D2       | zenerova dioda 5V6 | [BZX55C5V6.pdf](http://www.tme.eu/cz/Document/93af4cbc03c9aec362b15e73d9dbd6f8/BZX55Cx.pdf)             | [tme.eu](http://www.tme.eu/cz/details/bzx55c5v6/zenerovy-diody-tht/dc-components/)                     |

# Fotky

![Digitální teploměr](teplomer_2.jpg?width=300)
![Digitální teploměr](teplomer_3.jpg?width=300)
![Digitální teploměr](teplomer_4.jpg?width=300)
![Zapojení DS18B20](ds18b20-pinout.jpg)

# Zdrojový kód

Jen hlavní soubor `digitalni_teplomer.c`, pokud si chcete projekt zkompilovat sami, stáhněte si [celý projekt](https://git.ok1kvk.cz/krouzek/digitalni-teplomer-s-atmega8/repository/archive.zip?ref=master) se všemi soubory.

```
#include <avr/io.h>
#include <avr/interrupt.h>

#include "ds18b20_1.h"
#include "macros.h"

//Definice pozic segmentů na pinech portu
#define _s_A   PD2
#define _s_B   PD0
#define _s_C   PD6
#define _s_D   PD4
#define _s_E   PD3
#define _s_F   PD1
#define _s_G   PD7
#define _s_dot PD5

const unsigned char segs[] = //Definice číslic na 7segmentové displeji
{
    _BV(_s_A) | _BV(_s_B) | _BV(_s_C) | _BV(_s_D) | _BV(_s_E) | _BV(_s_F),              //0
    _BV(_s_B) | _BV(_s_C),                                                              //1
    _BV(_s_A) | _BV(_s_B) | _BV(_s_D) | _BV(_s_E) | _BV(_s_G),                          //2
    _BV(_s_A) | _BV(_s_B) | _BV(_s_C) | _BV(_s_D) | _BV(_s_G),                          //3
    _BV(_s_B) | _BV(_s_C) | _BV(_s_F) | _BV(_s_G),                                      //4
    _BV(_s_A) | _BV(_s_C) | _BV(_s_D) | _BV(_s_F) | _BV(_s_G),                          //5
    _BV(_s_A) | _BV(_s_C) | _BV(_s_D) | _BV(_s_E) | _BV(_s_F) | _BV(_s_G),              //6
    _BV(_s_A) | _BV(_s_B) | _BV(_s_C),                                                  //7
    _BV(_s_A) | _BV(_s_B) | _BV(_s_C) | _BV(_s_D) | _BV(_s_E) | _BV(_s_F) | _BV(_s_G),  //8
    _BV(_s_A) | _BV(_s_B) | _BV(_s_C) | _BV(_s_F) | _BV(_s_G),                          //9
    _BV(_s_G)                                                                           //mínus
};

const unsigned char err[] = //Chyba
{
    0,                                                          //mezera
    _BV(_s_A) | _BV(_s_D) | _BV(_s_E) | _BV(_s_F) | _BV(_s_G),  //E
    _BV(_s_E) | _BV(_s_G),                                      //r
    _BV(_s_E) | _BV(_s_G)                                       //r
};

volatile int temp; //Proměnná pro teplotu

//V tomto přerušení se zobrazuje na despleji
ISR(TIMER1_COMPA_vect)
{
    static int temp_1=0;
    static unsigned char disp=0;
    unsigned char dec;


    disp++;
    if(disp>3) disp=0;          //cyklicky 0 - 4 (sedmisegmentovka)

    PORTD=0xff;                 //vypnout segmenty
    PORTC=0b00100000>>disp;     //přepnout na danou sedmisegmentovku

    if (temp==0x5fff)    //pokud je chyba čidla
    {
        PORTD=(~err[disp]);    //vypsání znaku chyby
    }

    else //pokud není chyba čidla
    {
        switch(disp)
        {
            case 0://1.cifra - znam�nko
                    if (temp>>15) //pokud je nejvyšší bit 1 (záporné číslo)
                    {
                        temp_1=temp*(-1);    //převedení záporné teploty na kladnou
                        PORTD=(~segs[10]);    //zobrazení mínus
                    }
                    else temp_1=temp/16;        //převedení kladné teploty
                break;

            case 1://2.cifra - desítky
                    dec = temp_1 / 10;

                    if(dec>0) PORTD=(~segs[dec]);//zobrazit desítky
                break;

            case 2://3.cifra - jednotky
                    PORTD=(~segs[temp_1 % 10]) & ~_BV(_s_dot); //zobrazit jednotky a desetinnou tečku
                break;

            case 3://4.cifra - desetiny
                    PORTD = (~segs[((temp % 16)*625)/1000]); //zobrazit desetiny
                break;
        }//switch(disp)
    }//else (temp==0x5fff)
}//ISR(TIMER1_COMPA_vect)


int main(void)
{
    DDRD = 0xFF;        //výstupy - segmenty
    DDRC = 0x3F;        //výstupy - displeje

    setb(TCCR1B,0);
    setb(TCCR1B,1);     //start čítače 1 s předděličkou 64
    setb(TCCR1B,3);     //CTC mode

    OCR1AH=0x01;        //porovnávací registry
    OCR1AL=0x39;        //0x0139 = 313 = 8 000 000 / 64 / (100 * 4)  (=> 100 Hz)

    setb(TIMSK,4);      //povolení přerušení od šasovače 1
    setb(SREG,7);       //globální povolení přerušení

    while(1)
    {
         temp=gettemp();//přečtení teploty z čidla
    }
}
```
