```
title = "Vlastní tlačítka do Eagle CAD"
perex = "Oblíbený program Eagle má nové majitele a zatím to vypadá, že Eagle čeká mnoho nového, které především zefektivní práci. Ale není třeba čekat co přinesou vývojáři z Autodesku, můžete si jej vylepšit i sami. V následujícím článku si popíšeme jak si vytvořit vlastní tlačítka a jak definovat jejich funkce."
tags = ["Články","Technické Články","Eagle"]
lang = "cpp"
image= "eagle_lead1.jpg"
[author]
name = "Michal, OK1WMR"
[date]
creation = "2017-03-23T22:00"
```

Oblíbený program Eagle má nové majitele a zatím to vypadá, že Eagle čeká mnoho nového, které především zefektivní práci. Ale není třeba čekat co přinesou vývojáři z Autodesku, můžete si jej vylepšit i sami. V následujícím článku si popíšeme jak si vytvořit vlastní tlačítka a jak definovat jejich funkce.

Protože v minulých letech se jednotlivé verze Eaglu 5, 6, 7 od sebe moc nelišily. Uživatelé sami tedy hledali možnosti jak si Egale alespoň trochu přizpůsobit. Jednou z hlavních možností je vytvoření vlastního tlačítka. Zde je postup.

# Úprava eagle.csr

K tomu abyste mohli tlačítko přidat, musíte nejprve najít soubor eagle.scr, ten je vhodné nejprve zazálohovat. My ho máme umístěn na radioklubovém serveru ve složce EAGLE/SCRs.

![Hledáme eagle.scr](sw_eagle_01.png)

Tento soubor obsahuje pro každou část programu (BRD, SCH, LBR, DEV, SYM, PAC) do které je možné tlačítko naprogramovat. Nás bude zajímat především BRD: (Board) a SCH: (Schematic), ty používáme nejčastěji.
Tlačítko je pak vlastní definice cesty k grafické ikoně, názvu (zobrazuje se při umístění kurzoru na tlačítko) a vlastní funkce.

# Příklad přidání tlačítka

do části BRD: MENU přidáme tento kód:

    '[S:\EAGLE\SCRs\ikony_do_bin\ripup.png] Ripup all polygons : ripup @;'\

tento kód použije ikonu _ripup.png_ z nadefinovaného adresáře a pomocí příkazu _ripup @;_ zruší jedním klinutím vykreslení všech polygonů

## Výsledná funkce tlačítka

![výsledek](ripup.gif)

# Možnosti využití

- přednastavené přepínání vrstev a jejich zobrazení
- snadná volba a zobrazení definovaných rastrů
- rychlé přidání základních součástek a bloků
- spouštení oblíbených scriptů přes tlačítka
- zrychlené volby funkce CHANGE

## Příklad našeho EAGLE.SCR v39 (03/2017)

    # Configuration Script
    #
    # This file can be used to configure the editor windows.
    #
    # Uncomment this if you want a set of useful default shortcuts!
    #SCRIPT default-assign.scr;
    #
    BRD:
    MENU
    '[S:\EAGLE\SCRs\ikony_do_bin\mrizka.png] Mřížku chci {\
    	0,079375 mm : GRID ON;GRID LINES;GRID MM;GRID 0,079375 ; |\
    	0.15875 mm : GRID ON;GRID LINES;GRID MM;GRID 0.15875; |\
     	0.3175 mm : GRID ON;GRID LINES;GRID MM;GRID 0.3175 ; |\
     	0.635 mm  : GRID ON;GRID LINES;GRID MM;GRID 0.635; ; |\
        1.27 mm   : GRID ON;GRID LINES;GRID MM;GRID 1.27;    |\
        2.54 mm   : GRID ON;GRID LINES;GRID MM;GRID 2.54;    |\
        5.08 mm   : GRID ON;GRID LINES;GRID MM;GRID 5.08;    |\
        1 mm      : GRID ON;GRID LINES;GRID MM;GRID 1;        \
    }'\
    '[S:\EAGLE\SCRs\ikony_do_bin\zobraz.png] Zobraz     {\
    	vrstvu spojů TOP : display none; display 1 17 18 19 20 21 23 25 51; |\
    	vrstvu spojů BOT : display none; display 16 17 18 19 20 22 24 26 52; |\
        obě vrstvy spojů : display none; display 1 16 17 18 19 20 21 22 23 24 25 26 51 52; |\
        osazovák TOP: display none; display 1 17 18 19 20 21 25 51; SET COLOR_LAYER 1 17; SET COLOR_LAYER 16 17; SET COLOR_LAYER 17 15;SET COLOR_LAYER 18 15;SET COLOR_LAYER 21 15;SET COLOR_LAYER 25 15;SET COLOR_LAYER 51 15; |\
        osazovák BOT: display none; display 16 17 18 19 20 22 26 52; SET COLOR_LAYER 16 17; SET COLOR_LAYER 16 17; SET COLOR_LAYER 17 15;SET COLOR_LAYER 18 15;SET COLOR_LAYER 22 15;SET COLOR_LAYER 26 15;SET COLOR_LAYER 52 15; |\
        původní barvy: SET COLOR_LAYER 1 4; SET COLOR_LAYER 16 1; SET COLOR_LAYER 17 2; SET COLOR_LAYER 18 2; SET COLOR_LAYER 19 6; SET COLOR_LAYER 20 15; SET COLOR_LAYER 21 7; SET COLOR_LAYER 22 7; SET COLOR_LAYER 25 7; SET COLOR_LAYER 26 7; SET COLOR_LAYER 51 7; SET COLOR_LAYER 52 7;\
    }'\
        '[S:\EAGLE\SCRs\ikony_do_bin\chwidth.png] Změň WIDTH : CHANGE WIDTH'\
        '[S:\EAGLE\SCRs\ikony_do_bin\pad.png] čísla padů {\
     			ON : SET PAD_NAMES ON ; |\
                OFF: SET PAD_NAMES OFF;  \
    }'\
        '[S:\EAGLE\SCRs\ikony_do_bin\chfont.png] Změň FONT : CHANGE FONT VECTOR, CHANGE SIZE 1mm '\
        '[S:\EAGLE\SCRs\ikony_do_bin\chsize.png] Změň SIZE : CHANGE SIZE'\
        '[S:\EAGLE\SCRs\ikony_do_bin\chratio.png] Změň RATIO : CHANGE RATIO'\
        '[S:\EAGLE\SCRs\ikony_do_bin\delky.png] Délky vodičů : run length-freq-ri;'\
        '[S:\EAGLE\SCRs\ikony_do_bin\ripup.png] Ripup all polygons : ripup @;'\
    ;

    SCH:
    Grid Default;
    Change Width 0.006;
    MENU
    '[S:\EAGLE\SCRs\ikony_do_bin\mrizka.png] Mřížku chci {\
        	0.3175 mm : GRID ON;GRID LINES;GRID MM;GRID 0.3175 ; |\
     		0.635 mm  : GRID ON;GRID LINES;GRID MM;GRID 0.635; ; |\
     		1.27 mm   : GRID ON;GRID LINES;GRID MM;GRID 1.27;    |\
     		2.54 mm   : GRID ON;GRID LINES;GRID MM;GRID 2.54;    |\
     		5.08 mm   : GRID ON;GRID LINES;GRID MM;GRID 5.08;    |\
     		1 mm      : GRID ON;GRID LINES;GRID MM;GRID 1;        \
     		}'\
    '[S:\EAGLE\SCRs\ikony_do_bin\gnd.png] Vlož zem : ADD GND@#Paja_31'\
    '[S:\EAGLE\SCRs\ikony_do_bin\5v.png]  Vlož napětí 	{\
     		+3V3 : ADD +3v3@supply1  |\
            +5V  : ADD +5v@supply1  |\
     		+12V : ADD +12v@supply1 |\
     		+ Vcc : ADD vcc@supply1 \
     		}'\
    '[S:\EAGLE\SCRs\ikony_do_bin\r.png]  Přidej R 	{\
     			R-0805 : add R-EU_R0805@rcl  |\
     			R-0603 : add R-EU_R0603@rcl  |\
                R-0402 : add R-EU_R0402@rcl   \
            }   '\
    '[S:\EAGLE\SCRs\ikony_do_bin\c_ker.png]  Přidej C 	{\
     			C-0805 : add C-EUC0805@rcl  |\
     			C-0603 : add C-EUC0603@rcl  |\
                C-0402 : add C-EUC0402@rcl   \
                }'\
    '[S:\EAGLE\SCRs\ikony_do_bin\c_pol.png]  Přidej C-POL 	{\
    	Pouzdro A : add CPOL-EUSMCA@rcl  |\
        Pouzdro B : add CPOL-EUSMCB@rcl  |\
        Pouzdro C : add CPOL-EUSMCC@rcl  |\
        Pouzdro D : add CPOL-EUSMCD@rcl  |\
        Pouzdro E : add CPOL-EUSMCE@rcl   \
        }'\
    '[S:\EAGLE\SCRs\ikony_do_bin\ramecek.png] Vlož rámeček {\
    		A4 na šířku : ADD DINA4_L |\
    		A4 na výšku : ADD DINA4_P |\
            A3 na šířku : ADD DINA3_L |\
            A3 na výšku : ADD DINA3_P  \
            }'\
    '[S:\EAGLE\SCRs\ikony_do_bin\renumber.png] Přejmenuj názvy součástek : run renumber-sheet.ulp'\
    ;

## Rychlé přepínání barev k jednotlivým vrstvám

Osazovák TOP a Osazovák BOT - tuto funkci používáme pro rychlou změnu nastavená barev jednotlivých vrstev pro tisk/export přehledných nebo černobílích podkladů pro osazovací plán desky plošných spojů (v editoru desky). Díky inverznímu nastavení barev je pak plošný spoj světle šedý a popisky černé.

![Osazovák](sw_eagle_03.png?width=400)

# Kde brát inspiraci

- [můj oblíbený Eagle HELP na web.mit.edu/xavid](http://web.mit.edu/xavid/arch/i386_rhel4/help/24.htm)
- [Oficiální EAGLE dokumentace od Autodesku](http://eagle.autodesk.com/eagle/documentation)

# Dejte nám vědět!

Pokud máte zajímavý nápad na tlafíčka či funkci, napište nám na naše [fórum](http://forum.ok1kvk.cz/).
